@@ -1,46 +1,53 @@
 package org.apache.velocity;
 
 /*
- * Copyright 2000-2001,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- * 
- *      http://www.apache.org/licenses/LICENSE-2.0
- * 
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
-import java.io.InputStream;
-import java.io.Writer;
 import java.io.BufferedReader;
+import java.io.IOException;
+import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.UnsupportedEncodingException;
+import java.io.Writer;
+import java.util.List;
 
-import org.apache.velocity.runtime.resource.Resource;
-import org.apache.velocity.runtime.parser.ParseException;
-import org.apache.velocity.runtime.parser.node.SimpleNode;
-
-import org.apache.velocity.VelocityContext;
 import org.apache.velocity.context.Context;
 import org.apache.velocity.context.InternalContextAdapterImpl;
-
-import org.apache.velocity.exception.ResourceNotFoundException;
-import org.apache.velocity.exception.ParseErrorException;
 import org.apache.velocity.exception.MethodInvocationException;
+import org.apache.velocity.exception.ParseErrorException;
+import org.apache.velocity.exception.ResourceNotFoundException;
+import org.apache.velocity.exception.TemplateInitException;
+import org.apache.velocity.exception.VelocityException;
+import org.apache.velocity.runtime.parser.ParseException;
+import org.apache.velocity.runtime.parser.node.SimpleNode;
+import org.apache.velocity.runtime.resource.Resource;
+import org.apache.velocity.runtime.resource.ResourceManager;
 
 /**
  * This class is used for controlling all template
  * operations. This class uses a parser created
  * by JavaCC to create an AST that is subsequently
- * traversed by a Visitor. 
+ * traversed by a Visitor.
  *
  * <pre>
+ * // set up and initialize Velocity before this code block
+ *
  * Template template = Velocity.getTemplate("test.wm");
  * Context context = new VelocityContext();
  *
@@ -52,22 +59,18 @@ import org.apache.velocity.exception.MethodInvocationException;
  *
  * @author <a href="mailto:jvanzyl@apache.org">Jason van Zyl</a>
  * @author <a href="mailto:geirm@optonline.net">Geir Magnusson Jr.</a>
- * @version $Id: Template.java,v 1.36.4.1 2004/03/03 22:28:24 geirm Exp $
+ * @version $Id: Template.java 685724 2008-08-13 23:12:12Z nbubna $
  */
 public class Template extends Resource
 {
-    /**
-     *   To keep track of whether this template has been
-     *   initialized. We use the document.init(context)
-     *   to perform this.
-     */
-    private boolean initialized = false;
-
-    private Exception errorCondition = null;
+    private VelocityException errorCondition = null;
 
     /** Default constructor */
     public Template()
     {
+        super();
+        
+        setType(ResourceManager.RESOURCE_TEMPLATE);
     }
 
     /**
@@ -78,11 +81,10 @@ public class Template extends Resource
      *          from any available source.
      * @throws ParseErrorException if template cannot be parsed due
      *          to syntax (or other) error.
-     * @throws Exception some other problem, should only be from 
-     *          initialization of the template AST.
+     * @throws IOException problem reading input stream
      */
     public boolean process()
-        throws ResourceNotFoundException, ParseErrorException, Exception
+        throws ResourceNotFoundException, ParseErrorException, IOException
     {
         data = null;
         InputStream is = null;
@@ -91,7 +93,7 @@ public class Template extends Resource
         /*
          *  first, try to get the stream from the loader
          */
-        try 
+        try
         {
             is = resourceLoader.getResourceStream(name);
         }
@@ -119,14 +121,13 @@ public class Template extends Resource
             try
             {
                 BufferedReader br = new BufferedReader( new InputStreamReader( is, encoding ) );
- 
                 data = rsvc.parse( br, name);
                 initDocument();
                 return true;
             }
             catch( UnsupportedEncodingException  uce )
-            {   
-                String msg = "Template.process : Unsupported input encoding : " + encoding 
+            {
+                String msg = "Template.process : Unsupported input encoding : " + encoding
                 + " for template " + name;
 
                 errorCondition  = new ParseErrorException( msg );
@@ -137,30 +138,32 @@ public class Template extends Resource
                 /*
                  *  remember the error and convert
                  */
-
-               errorCondition =  new ParseErrorException( pex.getMessage() );
-               throw errorCondition;
+                errorCondition =  new ParseErrorException( pex );
+                throw errorCondition;
             }
-            catch( Exception e )
+            catch ( TemplateInitException pex )
             {
-                /*
-                 *  who knows?  Something from initDocument()
-                 */
-
-                errorCondition = e;
-                throw e;
+                errorCondition = new ParseErrorException( pex );
+                throw errorCondition;
             }
-            finally 
+            /**
+             * pass through runtime exceptions
+             */
+            catch( RuntimeException e )
+            {
+                throw new RuntimeException("Exception thrown processing Template "+getName(), e);
+            }
+            finally
             {
                 /*
                  *  Make sure to close the inputstream when we are done.
                  */
                 is.close();
             }
-        }    
+        }
         else
         {
-            /* 
+            /*
              *  is == null, therefore we have some kind of file issue
              */
 
@@ -170,18 +173,19 @@ public class Template extends Resource
     }
 
     /**
-     *  initializes the document.  init() is not longer 
-     *  dependant upon context, but we need to let the 
+     *  initializes the document.  init() is not longer
+     *  dependant upon context, but we need to let the
      *  init() carry the template name down throught for VM
      *  namespace features
+     * @throws TemplateInitException When a problem occurs during the document initialization.
      */
     public void initDocument()
-        throws Exception
+    throws TemplateInitException
     {
         /*
          *  send an empty InternalContextAdapter down into the AST to initialize it
          */
-        
+
         InternalContextAdapterImpl ica = new InternalContextAdapterImpl(  new VelocityContext() );
 
         try
@@ -191,7 +195,8 @@ public class Template extends Resource
              */
 
             ica.pushCurrentTemplateName( name );
-            
+            ica.setCurrentResource( this );
+
             /*
              *  init the AST
              */
@@ -200,22 +205,20 @@ public class Template extends Resource
         }
         finally
         {
-            /*  
+            /*
              *  in case something blows up...
              *  pull it off for completeness
              */
 
             ica.popCurrentTemplateName();
+            ica.setCurrentResource( null );
         }
 
     }
 
     /**
      * The AST node structure is merged with the
-     * context to produce the final output. 
-     *
-     * Throws IOException if failure is due to a file related
-     * issue, and Exception otherwise
+     * context to produce the final output.
      *
      *  @param context Conext with data elements accessed by template
      *  @param writer output writer for rendered template
@@ -223,14 +226,37 @@ public class Template extends Resource
      *          from any available source.
      *  @throws ParseErrorException if template cannot be parsed due
      *          to syntax (or other) error.
-     *  @throws  Exception  anything else. 
+     *  @throws MethodInvocationException When a method on a referenced object in the context could not invoked.
+     *  @throws IOException  Might be thrown while rendering.
      */
     public void merge( Context context, Writer writer)
-        throws ResourceNotFoundException, ParseErrorException, MethodInvocationException, Exception
+        throws ResourceNotFoundException, ParseErrorException, MethodInvocationException, IOException
+    {
+        merge(context, writer, null);
+    }
+
+    
+    /**
+     * The AST node structure is merged with the
+     * context to produce the final output.
+     *
+     *  @param context Conext with data elements accessed by template
+     *  @param writer output writer for rendered template
+     *  @param macroLibraries a list of template files containing macros to be used when merging
+     *  @throws ResourceNotFoundException if template not found
+     *          from any available source.
+     *  @throws ParseErrorException if template cannot be parsed due
+     *          to syntax (or other) error.
+     *  @throws MethodInvocationException When a method on a referenced object in the context could not invoked.
+     *  @throws IOException  Might be thrown while rendering.
+     *  @since 1.6
+     */
+    public void merge( Context context, Writer writer, List macroLibraries)
+        throws ResourceNotFoundException, ParseErrorException, MethodInvocationException, IOException
     {
         /*
-         *  we shouldn't have to do this, as if there is an error condition, 
-         *  the application code should never get a reference to the 
+         *  we shouldn't have to do this, as if there is an error condition,
+         *  the application code should never get a reference to the
          *  Template
          */
 
@@ -248,6 +274,52 @@ public class Template extends Resource
 
             InternalContextAdapterImpl ica = new InternalContextAdapterImpl( context );
 
+            /**
+             * Set the macro libraries
+             */
+            ica.setMacroLibraries(macroLibraries);
+
+            if (macroLibraries != null)
+            {
+                for (int i = 0; i < macroLibraries.size(); i++)
+                {
+                    /**
+                     * Build the macro library
+                     */
+                    try
+                    {
+                        rsvc.getTemplate((String) macroLibraries.get(i));
+                    }
+                    catch (ResourceNotFoundException re)
+                    {
+                        /*
+                        * the macro lib wasn't found.  Note it and throw
+                        */
+                        rsvc.getLog().error("template.merge(): " +
+                                "cannot find template " +
+                                (String) macroLibraries.get(i));
+                        throw re;
+                    }
+                    catch (ParseErrorException pe)
+                    {
+                        /*
+                        * the macro lib was found, but didn't parse - syntax error
+                        *  note it and throw
+                        */
+                        rsvc.getLog().error("template.merge(): " +
+                                "syntax error in template " +
+                                (String) macroLibraries.get(i) + ".");
+                        throw pe;
+                    }
+                    
+                    catch (Exception e)
+                    {
+                        throw new RuntimeException("Template.merge(): parse failed in template  " +
+                                (String) macroLibraries.get(i) + ".", e);
+                    }
+                }
+            }
+
             try
             {
                 ica.pushCurrentTemplateName( name );
@@ -258,7 +330,7 @@ public class Template extends Resource
             finally
             {
                 /*
-                 *  lets make sure that we always clean up the context 
+                 *  lets make sure that we always clean up the context
                  */
                 ica.popCurrentTemplateName();
                 ica.setCurrentResource( null );
@@ -270,13 +342,11 @@ public class Template extends Resource
              * this shouldn't happen either, but just in case.
              */
 
-            String msg = "Template.merge() failure. The document is null, " + 
+            String msg = "Template.merge() failure. The document is null, " +
                 "most likely due to parsing error.";
 
-            rsvc.error(msg);
-            throw new Exception(msg);
+            throw new RuntimeException(msg);
+
         }
     }
 }
-
-
