@@ -111,7 +111,7 @@ import org.apache.velocity.util.introspection.LinkingUberspector;
  * @author <a href="mailto:jvanzyl@apache.org">Jason van Zyl</a>
  * @author <a href="mailto:jlb@houseofdistraction.com">Jeff Bowden</a>
  * @author <a href="mailto:geirm@optonline.net">Geir Magusson Jr.</a>
- * @version $Id: RuntimeInstance.java 703049 2008-10-09 03:18:58Z nbubna $
+ * @version $Id: RuntimeInstance.java 896109 2010-01-05 15:59:48Z nbubna $
  */
 public class RuntimeInstance implements RuntimeConstants, RuntimeServices
 {
@@ -141,7 +141,7 @@ public class RuntimeInstance implements RuntimeConstants, RuntimeServices
     /**
      * Indicate whether the Runtime has been fully initialized.
      */
-    private boolean initialized = false;
+    private volatile boolean initialized = false;
 
     /**
      * These are the properties that are laid down over top
@@ -282,7 +282,7 @@ public class RuntimeInstance implements RuntimeConstants, RuntimeServices
      */
     private void requireInitialization()
     {
-        if (!initialized && !initializing)
+        if (!initialized)
         {
             log.debug("Velocity was not initialized! Calling init()...");
             try
